package endrov.imageset;

import loci.common.DataTools;
import loci.formats.FormatTools;
import loci.formats.IFormatReader;

/**
 * Image I/O
 */
public class BioformatsSliceIO implements EvIOImage
	{
	private int id;
	//private Integer subid;
	private IFormatReader imageReader;
	private String sourceName;

	
	public BioformatsSliceIO(IFormatReader imageReader, int id, Integer subid, String sourceName)
		{
		this.imageReader=imageReader;
		this.id=id;
		//this.subid=subid;
		this.sourceName=sourceName;
		}

	
	public String sourceName()
		{
		return sourceName;
		}

	
	
	/**
	 * Load the image
	 */
	public EvPixels loadJavaImage()
		{
		try
			{
			
			
			byte[] bytes=imageReader.openBytes(id);
			
			//FormatTools, DOUBLE, FLOAT, INT16, INT32, INT8, UINT16, UINT32, UINT8
			
			
			int w=imageReader.getSizeX();
			int h=imageReader.getSizeY();
			//DataInputStream di=new DataInputStream(new ByteArrayInputStream(bytes));
			
			int type=imageReader.getPixelType();
			int bpp=FormatTools.getBytesPerPixel(type);
			boolean isFloat = type == FormatTools.FLOAT || type == FormatTools.DOUBLE;
			boolean isLittle = imageReader.isLittleEndian();
			boolean isSigned = type == FormatTools.INT8 || type == FormatTools.INT16 || type == FormatTools.INT32;
			Object bfpixels = DataTools.makeDataArray(bytes, bpp, isFloat, isLittle);
			
			
			//System.out.println(bfpixels.getClass()+"  "+isSigned);
			
			//Much of this code modified from bioformats IJ-plugin. I deem it functional and hence not copyrightable
			//https://skyking.microscopy.wisc.edu/trac/java/browser/trunk/loci/plugins/Util.java?rev=4289
			//unsigned values have to be upconverted to fit into signed
			if (bfpixels instanceof byte[]) 
				{
				byte[] q = (byte[]) bfpixels;
				int len=w*h;
				if(isSigned)
					{
					if (q.length > w * h) 
						{
						byte[] tmp = q;
						q = new byte[w * h];
						System.arraycopy(tmp, 0, q, 0, q.length);
						}
					q = DataTools.makeSigned(q);
					return EvPixels.createFromUByte(w, h, q);
					}
				else
					return EvPixels.createFromShort(w, h, EvPixels.convertUbyteToShort(q, len));
				}
			else if (bfpixels instanceof short[])
				{
				short[] q = (short[]) bfpixels;
				
				int len=w*h;
				if(isSigned)
					{
					if (q.length > w * h) 
						{
						short[] tmp = q;
						q = new short[w * h];
						System.arraycopy(tmp, 0, q, 0, q.length);
						}
					q = DataTools.makeSigned(q);
					return EvPixels.createFromShort(w, h, q);
					}
				else
					return EvPixels.createFromInt(w, h, EvPixels.convertUshortToInt(q, len));
				/*
				if (q.length > w * h) 
					{
					short[] tmp = q;
					q = new short[w * h];
					System.arraycopy(tmp, 0, q, 0, q.length);
					}

				if (isSigned) 
					q = DataTools.makeSigned(q);

				
				//TODO unsigned - upconvert
				return EvPixels.createFromShort(w, h, q);*/
				}
			else if (bfpixels instanceof int[])
				{
				int[] q = (int[]) bfpixels;
				if (q.length > w * h) 
					{
					int[] tmp = q;
					q = new int[w * h];
					System.arraycopy(tmp, 0, q, 0, q.length);
					}

				if (isSigned) 
					q = DataTools.makeSigned(q);

				//unsigned? - screw it. would have to convert to float/double, evil.
				return EvPixels.createFromInt(w, h, q);
				}
			else if (bfpixels instanceof float[])
				{
				float[] q = (float[]) bfpixels;
				if (q.length > w * h) 
					{
					float[] tmp = q;
					q = new float[w * h];
					System.arraycopy(tmp, 0, q, 0, q.length);
					}
				return EvPixels.createFromFloat(w, h, q);
				}
			else if (bfpixels instanceof double[]) 
				{
				double[] q = (double[]) bfpixels;
				if (q.length > w * h) 
					{
					double[] tmp = q;
					q = new double[w * h];
					System.arraycopy(tmp, 0, q, 0, q.length);
					}
				return EvPixels.createFromDouble(w, h, q);
				}
			else
				{
				System.out.println("Bioformats returns unrecognized format");
				System.out.println(bfpixels.getClass()+"  "+isSigned);
				return null;
				}

			}
		catch(Exception e)
			{
			endrov.ev.EvLog.printError("Failed to read image "+sourceName(),e);
			return null;
			}
		}
	}